//  .----------------.  .----------------.  .----------------.  .----------------.  .-----------------.
// | .--------------. || .--------------. || .--------------. || .--------------. || .--------------. |
// | |   _____      | || |   ______     | || | ____    ____ | || |      __      | || | ____  _____  | |
// | |  |_   _|     | || |  |_   _ \    | || ||_   \  /   _|| || |     /  \     | || ||_   \|_   _| | |
// | |    | |       | || |    | |_) |   | || |  |   \/   |  | || |    / /\ \    | || |  |   \ | |   | |
// | |    | |   _   | || |    |  __'.   | || |  | |\  /| |  | || |   / ____ \   | || |  | |\ \| |   | |
// | |   _| |__/ |  | || |   _| |__) |  | || | _| |_\/_| |_ | || | _/ /    \ \_ | || | _| |_\   |_  | |
// | |  |________|  | || |  |_______/   | || ||_____||_____|| || ||____|  |____|| || ||_____|\____| | |
// | |              | || |              | || |              | || |              | || |              | |
// | '--------------' || '--------------' || '--------------' || '--------------' || '--------------' |
//  '----------------'  '----------------'  '----------------'  '----------------'  '----------------' 
//
// https://mmg-gitlab.fjfi.cvut.cz/gitlab/lbm/lbman
//
// Written by Radek Fučík (fucik@fjfi.cvut.cz) FNSPE CTU in Prague, Czech Republic
// with a great help of Robert Straka (straka@metal.agh.edu.pl) AGH Krakow, Poland
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// https://www.gnu.org/licenses/

#include "defs.h"
#include "core.h"

void execute(
	int DIM = 3,			// dimension
	int Q = 7,			// number of discrete velocities
	int model = SRT,		// collision model
	bool constvel = true,		// velocity field is constant in time and space (for ADE)
	bool bigprint = false,		// use a more detailed decomposition of exported coefficients
	bool verbosity = true,
	bool EFDEprint = true,		// export EFDE
	bool EPDEprint = true,		// export EPDE
	bool factor_coefs = false,	// use GiNaC::factor() when exporting coefficients to latex
	int feqorder = 2, 		// -1 = symbolic, 1 = linear, 2 = quadratic
	int ORDER = 2			// Taylor expansion order
)
{
	printf("d=%d q=%d model=%d feqorder=%d constvel=%s bigprint=%d\n", DIM, Q, model, feqorder, (constvel)?"yes":"no", bigprint);
	fflush(stdout);

	CORE core(DIM,Q);

	core.collision_model=model;
	core.verbosity=verbosity;
	core.constvel=constvel;
	core.feqorder=feqorder;
	core.bigprint=bigprint;
	core.EFDEprint=EFDEprint;
	core.factor_coefs=factor_coefs;
	core.ORDER = ORDER;
	core.sym_cs=(ex)1/sqrt((ex)3);

	core.allocate();
	sprintf(core.target_dir,"output_D%dQ%d%s",DIM,Q,(constvel)?"c":"n");
	if (verbosity) { printf("allocated\n"); fflush(stdout); }


	core.define_matrix_M();
	if (verbosity) { printf("M defined\n"); fflush(stdout); }
	
	core.define_matrix_A();
	if (verbosity) { printf("A defined\n"); fflush(stdout); }
	
	core.define_matrix_B();
	if (verbosity) { printf("B defined\n"); fflush(stdout); }

	core.define_equilibrium();
	if (verbosity) { printf("feq defined\n"); fflush(stdout); }

	core.compute_coefficients();
	if (verbosity) { printf("coefficients computed\n"); fflush(stdout); }

	core.compute_EFDE_coefs();
	if (verbosity) { printf("CORE coefs computed\n"); fflush(stdout); }

	if (EFDEprint) for (int d=0;d<Q;d++) core.export_EFDE_coefs(d);

	core.compute_EPDE_coefs();
	if (verbosity) { printf("EPDE coefs computed\n"); fflush(stdout); }

	if (EPDEprint) for (int d=0;d<Q;d++) core.export_EPDE_coefs(d);
}

void d1q3(bool constvel=true)
{
	execute(1,3,SRT,constvel);
	execute(1,3,MRT1,constvel);
	execute(1,3,CLBM1,constvel);
}

void d2q5(bool constvel=true)
{
	bool bigprint=!constvel;
	execute(2,5,SRT,constvel,bigprint);
	execute(2,5,MRT1,constvel,bigprint);
	execute(2,5,MRT2,constvel,bigprint);
	execute(2,5,MRT1_OG,constvel,bigprint);
	execute(2,5,MRT2_OG,constvel,bigprint);
	execute(2,5,CLBM1,constvel,bigprint);
	execute(2,5,CLBM2,constvel,bigprint);
}

void d3q7(bool constvel=true)
{
	bool bigprint=!constvel;
	execute(3,7,SRT,constvel,bigprint);
	execute(3,7,MRT1,constvel,bigprint);
	execute(3,7,MRT2,constvel,bigprint);
	execute(3,7,MRT1_OG,constvel,bigprint);
	execute(3,7,MRT2_OG,constvel,bigprint);
	execute(3,7,CLBM1,constvel,bigprint);
	execute(3,7,CLBM2,constvel,bigprint);
}

int main(int argc, char* argv[]) 
{
	d1q3(true);
//	d1q3(false);

	d2q5(true);
//	d2q5(false);

//	d3q7(true);
//	d3q7(false);
	return 0;
}


